@@ -31,27 +31,40 @@ typedef struct {
 	char       *original_path;    /* path read from command line. */
 	char       *full_path;        /* "/" + original_path. */
 	char       *link;
-	guint64     size;
+	goffset     size;
 	time_t      modified;
 
 	char       *name;             /* The file name. */
 	char       *path;             /* The directory. */
-	const char *type;             /* The mime type. */
-
+	gboolean    encrypted;        /* Whether the file is encrypted. */
+	gboolean    dir;              /* Whether this is a directory listed in the archive */
+	goffset     dir_size;
+	const char *content_type;
+	
 	/* Additional data. */
 
-	gboolean    is_dir;           /* Whether this entry is used to show
+	gboolean    list_dir;         /* Whether this entry is used to show
 				       * a directory. */
 	char       *list_name;        /* The string visualized in the list
 				       * view. */
+
+	/* Private data */
+
+	gboolean    free_original_path;
 } FileData;
 
 #define FR_TYPE_FILE_DATA (file_data_get_type ())
 
-GType           file_data_get_type             (void);
-FileData *      file_data_new                  (void);
-FileData *      file_data_copy                 (FileData *src);
-void            file_data_free                 (FileData *fdata);
-const char *    file_data_get_type_description (const FileData *fdata);
+GType           file_data_get_type            (void);
+FileData *      file_data_new                 (void);
+FileData *      file_data_copy                (FileData *src);
+void            file_data_free                (FileData *fdata);
+void            file_data_update_content_type (FileData *fdata);
+gboolean        file_data_is_dir              (FileData *fdata);
+
+int  file_data_compare_by_path                (gconstpointer a, 
+				               gconstpointer b);
+int  find_path_in_file_data_array             (GPtrArray   *array,
+				               const char *path);
 
 #endif /* FILE_DATA_H */
